/*  YAC: Yet Another Caller ID Program
 *  Copyright (C) 2002 Jensen Harris (jensen@sunflowerhead.com)
 *  http://www.sunflowerhead.com
 *
 *  YAC is free software; you can redistribute it and/or modify it under the
 *  terms of the GNU General Public License as published by the Free Software
 *  Foundation; either version 2 of the License, or (at your option) any
 *  later version.
 *
 *  YAC is distributed in the hope that it will be useful, but WITHOUT ANY
 *  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 *  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 *  details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with YAC; if not, write to the Free Software Foundation, Inc.,
 *  59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  File: yac.c
 *  Version: 0.17
 *  Last Modified: 17-Apr-07 by Matt Collinge to add external script support
 */

#include "yac.h"

YACINFO *g_yi;

int WINAPI WinMain(HINSTANCE hInst, HINSTANCE hPrevInst, LPSTR lpCmdLine, int nCmdShow)
{
	MSG msg;
	HANDLE hArray[2];
	YACINFO *yi;

	if (!EnsureNotRunning(hInst))
		return -1;

	// Create the one and only YACINFO struct
	yi = (YACINFO *) HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof(YACINFO));
	if (!yi)
		return -1;

	// Prepare the YACINFO struct for use by the rest of YAC
	InitYACInfoStruct(yi);
	g_yi = yi;
	yi->hinst = hInst;

	// Do all necessary program initialization
	if (!InitYAC(lpCmdLine, yi))
		return -1;

	// Main program message loop
	while (GetMessage(&msg, NULL, 0, 0))
	{
		if (!yi->hwndReadText || !IsDialogMessage(yi->hwndReadText, &msg))
		{
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
	}

	// Tell the threads that the party's over
	if (yi->nMode == MODE_SERVER)
	{
		SetEvent(yi->EventDisplayQuit);
		SetEvent(yi->EventPhoneListenQuit);
		lineShutdown(yi->hlapp);

		// Now, wait for them to gracefully terminate
		hArray[0] = yi->hPhoneThread;
		hArray[1] = yi->hDisplayThread;

		// If it takes longer than 5 seconds, we're in big trouble, just quit.
		WaitForMultipleObjects(2, hArray, TRUE, 5000);
	}
	else
	{
		SetEvent(yi->EventListenQuit);

		if (yi->serverFd)
			closesocket(yi->serverFd);

		WaitForSingleObject(yi->hListenThread, 5000);
	}

	CleanUpResources(yi);

	// Free the YACINFO struct
	HeapFree(GetProcessHeap(), 0, yi);

	return 0;
}


BOOL InitYAC(LPSTR lpCmdLine, YACINFO *yi)
{
	WSADATA wsaData;
	int nError;

	// Make a mutex so we know we're running
	yi->Mutex = CreateMutex(NULL, FALSE, YAC_MUTEX);

	// Create our hidden window
	yi->hwnd = CreateHiddenWindow(yi);

	if (!yi->hwnd)
		return FALSE;

	// Initialize the common controls
	InitCommonControls();

	// Find out if I'm a listener or a server
	if (!lstrcmpi(lpCmdLine, "listen"))
	{
		yi->nMode = MODE_LISTENER;
		strncpy(yi->szAppName, LISTENER_NAME, 32);
	}
	else
	{
		yi->nMode = MODE_SERVER;
		strncpy(yi->szAppName, SERVER_NAME, 32);
	}

	if (!ValidateOperatingSystem(yi))
		return FALSE;

	InitLogFile(yi);
		
	// Get configuration file
	GetIniFileName(yi->szIniFile);

	// Initialize the listeners data
	ReadListenersData(yi);

	// Initialize the system tray
	if (!InitSystemTray(yi))
		return FALSE;

	// Start Winsock
	nError = WSAStartup(MAKEWORD(2, 0), &wsaData);

	if (nError)
		return FALSE;

	// Seed the random number generator
	srand((unsigned)time(NULL));

	DoFirstRun(yi);
	CreateGDIResources(yi);
	GetRegistryOptions(yi);

	if (yi->nMode == MODE_SERVER)
	{
		// Create the neccesary events
		yi->EventNotify = CreateEvent(NULL, FALSE, FALSE, NULL);
		yi->EventDisplayQuit = CreateEvent(NULL, FALSE, FALSE, NULL);
		yi->EventPhoneListenQuit = CreateEvent(NULL, FALSE, FALSE, NULL);

		// Spawn phone listening thread
		yi->hPhoneThread = (HANDLE)_beginthreadex(NULL, 0,
			PhoneListenThread, NULL, 0, NULL);

		// Spawn phone listening thread for event notification
		yi->hDisplayThread = (HANDLE)_beginthreadex(NULL, 0,
			DisplayMessageThread, NULL, 0, NULL);
	}
	else
	{
		// Create events to notify the listening thread
		yi->EventListenQuit = CreateEvent(NULL, FALSE, FALSE, NULL);

		// Now, just sit back and listen for messages
		yi->hListenThread = (HANDLE)_beginthreadex(NULL, 0,
			ListenThread, NULL, 0, NULL);
	}
	return TRUE;
}


void CleanUpResources(YACINFO *yi)
{
	// Do necessary cleanup
	WSACleanup();
	yi->ni.uFlags = NIF_MESSAGE;
	Shell_NotifyIcon(NIM_DELETE, &yi->ni);

	if (yi->hDisplayThread)
		CloseHandle(yi->hDisplayThread);

	if (yi->hPhoneThread)
		CloseHandle(yi->hPhoneThread);

	if (yi->hNotifyNetThread)
		CloseHandle(yi->hNotifyNetThread);

	if (yi->hListenThread)
		CloseHandle(yi->hListenThread);

	if (yi->im)
		ImageList_Destroy(yi->im);

	if (yi->hfNotify)
		DeleteObject(yi->hfNotify);

	if (yi->hfNotifyUnderline)
		DeleteObject(yi->hfNotifyUnderline);

	if (yi->hfNotifyBold)
		DeleteObject(yi->hfNotifyBold);

	if (yi->hbmYak)
		DeleteObject(yi->hbmYak);

	if (yi->hbrBlack)
		DeleteObject(yi->hbrBlack);

	// Close handles and events
	if (yi->EventNotify)
		CloseHandle(yi->EventNotify);

	if (yi->EventDisplayQuit)
		CloseHandle(yi->EventDisplayQuit);
	
	if (yi->EventPhoneListenQuit)
		CloseHandle(yi->EventPhoneListenQuit);

	if (yi->EventNotifyNetQuit)
		CloseHandle(yi->EventNotifyNetQuit);

	if (yi->Mutex)
		CloseHandle(yi->Mutex);
}


void GetRegistryOptions(YACINFO *yi)
{
	BOOL fRet;
	TCHAR szText[256];

	// Get necessary options from the registry

	// The defaults should be built into this function.  Setup will
	// not write any registry keys from now on.

	if (yi->uOS >= OS_WIN2000)
		yi->fTranslucentWindows = TRUE;
	fRet = GetInfo("TranslucentWindows", szText, HKEY_LOCAL_MACHINE);
	if (fRet && (atoi(szText) == 0))
		yi->fTranslucentWindows = FALSE;		

	fRet = GetInfo("UseBalloons", szText, HKEY_LOCAL_MACHINE);
	if (fRet && (atoi(szText) == 1) && (yi->uOS >= OS_WINME))
		yi->fUseBalloons = TRUE;

	yi->fPlaySounds = TRUE;
	fRet = GetInfo("PlaySounds", szText, HKEY_LOCAL_MACHINE);
	if (fRet && (atoi(szText) == 0))
		yi->fPlaySounds = FALSE;

	fRet = GetInfo("ShareModem", szText, HKEY_LOCAL_MACHINE);
	yi->fShareModem = fRet && (atoi(szText) == 1);

	fRet = GetInfo("DisplayTime", szText, HKEY_LOCAL_MACHINE);
	
	yi->nDisplayTime = DEFAULT_DISPLAY_TIME;

	if (fRet)
		yi->nDisplayTime = atoi(szText);

	if (yi->nDisplayTime < MIN_DISPLAY_TIME || yi->nDisplayTime > MAX_DISPLAY_TIME)
		yi->nDisplayTime = DEFAULT_DISPLAY_TIME;

	fRet = GetInfo("Port", szText, HKEY_LOCAL_MACHINE);
	
	if (fRet)
	{
		yi->nServerPort = atoi(szText);

		if (yi->nServerPort < 1 || yi->nServerPort > 65536)
			yi->nServerPort = DEFAULT_PORT;
	}
	else
		yi->nServerPort = DEFAULT_PORT;
}


void CreateGDIResources(YACINFO *yi)
{
	// Fonts
	yi->hfNotify = GetFont("Arial", FALSE, FALSE, FALSE, 10, 0);
	yi->hfNotifyUnderline = GetFont("Arial", FALSE, FALSE, TRUE, 10, 0);
	yi->hfNotifyBold = GetFont("Arial", TRUE, FALSE, FALSE, 10, 0);

	// Bitmaps
	yi->hbmYak = LoadBitmap(yi->hinst, MAKEINTRESOURCE(IDB_YAC));

	// Sounds (OK, not really a GDI resource...)
	yi->szIncomingSound = MAKEINTRESOURCE(IDR_INCOMING);
	yi->szDoneSound = MAKEINTRESOURCE(IDR_DONE);
	
	// This call will always fail.  We try to play a sound that doesn't exist
	// so that the MMLIB DLL gets lazy loaded.  This prevents the pause the
	// first time we play a sound.
	PlaySound("", yi->hinst, SND_NODEFAULT);

	// Image list
	yi->im = ImageList_Create(16, 16, ILC_COLOR8, 2, 0);
	ImageList_SetBkColor(yi->im, GetSysColor(COLOR_WINDOW));
	ImageList_AddIcon(yi->im, (HICON)LoadImage(yi->hinst, MAKEINTRESOURCE(IDI_APP),
			IMAGE_ICON, 16, 16, LR_SHARED));
	ImageList_AddIcon(yi->im, (HICON)LoadImage(yi->hinst, MAKEINTRESOURCE(IDI_NUMBER),
			IMAGE_ICON, 16, 16, LR_SHARED));
	
	// Cursors
	yi->hcHand = LoadImage(NULL, IDC_HAND, IMAGE_CURSOR, LR_DEFAULTSIZE,
		LR_DEFAULTSIZE, LR_DEFAULTCOLOR | LR_SHARED);
	yi->hcArrow = LoadImage(NULL, IDC_ARROW, IMAGE_CURSOR, LR_DEFAULTSIZE,
		LR_DEFAULTSIZE, LR_DEFAULTCOLOR | LR_SHARED);

	// Brushes
	yi->hbrBlack = CreateSolidBrush(COLOR_BLACK);
}


void InitLogFile(YACINFO *yi)
{
	BOOL fRet;
	TCHAR szText[256], szText2[256], szText3[256];

	// Should we keep a log file?
	fRet = GetInfo("WriteLogFile", szText, HKEY_LOCAL_MACHINE);

	// Get log file name
	GetLogFileName(yi->szLogFile);

	yi->fWriteLogFile = TRUE;
	if (fRet && (atoi(szText) == 0))
		yi->fWriteLogFile = FALSE;

	// Should we write text messages to the log file?
	fRet = GetInfo("WriteTextToLogFile", szText, HKEY_LOCAL_MACHINE);

	if (fRet && (atoi(szText) == 1))
		yi->fWriteTextToLog = TRUE;

	// Write out the beginning of the log file
	LoadString(yi->hinst, IDS_LOGFILESTART, szText, 256);
	WriteLog(szText, yi);
	GetDateTimeString(szText, 256);
	LoadString(yi->hinst, IDS_LOGFILEVER, szText3, 256);
	_snprintf(szText2, 256, szText3, VERSION, szText);
	WriteLog(szText2, yi);

	if (yi->nMode == MODE_SERVER)
	{
		LoadString(yi->hinst, IDS_RUNSERVER, szText, 256);
		WriteLog(szText, yi);
	}
	else
	{
		LoadString(yi->hinst, IDS_RUNLISTENER, szText, 256);
		WriteLog(szText, yi);
	}
}


BOOL ValidateOperatingSystem(YACINFO *yi)
{
	OSVERSIONINFO os;
	TCHAR szText[256];
	BOOL fRet;

	// Find out what operating system we're running on
	ZeroMemory(&os, sizeof(os));
	os.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);

	GetVersionEx(&os);

	if (os.dwMajorVersion < 4)
		yi->uOS = OS_NOT_SUPPORTED;

	else if (os.dwMajorVersion == 4)
	{
		if (os.dwMinorVersion == 0)
		{
			if (os.dwPlatformId == VER_PLATFORM_WIN32_NT)
				yi->uOS = OS_NT4;
			else
				yi->uOS = OS_WIN95;
		}

		else if (os.dwMinorVersion == 10)
			yi->uOS = OS_WIN98;

		else if (os.dwMinorVersion == 90)
			yi->uOS = OS_WINME;
		else
			yi->uOS = OS_NOT_SUPPORTED;
	}
	else // (assume os.dwMajorVersion >= 5)
	{
		if (os.dwMajorVersion == 5 && os.dwMinorVersion == 0)
			yi->uOS = OS_WIN2000;
		else // assume 5.1 or greater
			yi->uOS = OS_WINXP_ABOVE;
	}

	if (yi->nMode == MODE_LISTENER && (yi->uOS == OS_NOT_SUPPORTED || yi->uOS == OS_WIN95))
	{
		LoadString(yi->hinst, IDS_LISTENEROS, szText, 256);
		MessageBox(NULL, szText, yi->szAppName, MB_ICONSTOP);
		return FALSE;
	}

	if ((yi->nMode == MODE_SERVER) && (yi->uOS < OS_WIN2000))
	{
		if (yi->uOS < OS_WINME)
			fRet = FALSE;
		else
			fRet = GetInfo("AllowWinMeServer", szText, HKEY_LOCAL_MACHINE);

		if (!fRet || (atoi(szText) != 1))
		{
			LoadString(yi->hinst, IDS_SERVEROS, szText, 256);
			MessageBox(NULL, szText, yi->szAppName, MB_ICONSTOP);
			return FALSE;
		}
	}

	return TRUE;
}


BOOL EnsureNotRunning(HINSTANCE hInst)
{
	HANDLE hMutex;
	TCHAR szText[256], szCaption[256];

	// Make sure YAC is not already running
	hMutex = OpenMutex(MUTEX_ALL_ACCESS, FALSE, YAC_MUTEX);

	if (hMutex)
	{
		CloseHandle(hMutex);
		LoadString(hInst, IDS_ALREADYRUNNING, szText, 256);
		LoadString(hInst, IDS_APPNAME, szCaption, 256);
		MessageBox(NULL, szText, szCaption, MB_ICONSTOP);
		return FALSE;
	}
	return TRUE;
}


void NotifyClients(YACINFO *yi)
{
	// Pass data along to the clients...
	SetEvent(yi->EventNotify);
}


unsigned __stdcall DisplayMessageThread(void* lpParameter)
{
	HANDLE hArray[2];
	DWORD dwReturn;
	TCHAR szName[32], szNumber[32];
	YACINFO *yi;
	CRITICAL_SECTION cs;

	GetYACInfoStruct(&yi);

	hArray[0] = yi->EventNotify;
	hArray[1] = yi->EventDisplayQuit;

	InitializeCriticalSection(&cs);

	while (TRUE)
	{
		// Wait until we hear something back from our event
		dwReturn = WaitForMultipleObjects(2, hArray, FALSE, INFINITE);

		if (1 == dwReturn - WAIT_OBJECT_0)
		{
			DeleteCriticalSection(&cs);
			return 0;
		}

		// OK, we must have been notified to display UI
		// Let's wait until the messages slow down, maybe we only need to show
		// the UI one time
		do
		{
			dwReturn = WaitForSingleObject(yi->EventNotify, MSG_WAIT_TIME);
		}
		while (dwReturn == WAIT_OBJECT_0);

		EnterCriticalSection(&cs);
		strcpy(szName, yi->szName);
		strcpy(szNumber, yi->szNumber);

		// Now, zero out the name and number so we don't use again
		yi->szName[0] = '\0';
		yi->szNumber[0] = '\0';

		LeaveCriticalSection(&cs);

		// Do the necessary formating of name and number strings
		DoNameSubstitution(szName, szNumber, yi);
		FormatNameAndNumber(szName, szNumber, yi);

		// Create the encoded transmission string
		EnterCriticalSection(&cs);
		EncodeCallerIDInfo(yi->szString, szName, szNumber);
		LeaveCriticalSection(&cs);

		// Forward on the finished string
		SendNotificationString(yi);
	}
}


void SendNotificationString(YACINFO *yi)
{
	int i;

	// Notify all the listeners, one at a time
	for (i = 0; i < yi->nNumListeners; i++)
		_beginthreadex(NULL, 0, NetConnectThread, (void *)i, 0, NULL);

	// Notify the local user
	SendMessage(yi->hwnd, YAC_MSG_NOTIFYUSER, (WPARAM)yi->szString, 0);
}


HWND CreateHiddenWindow(YACINFO *yi)
{
	WNDCLASSEX wcex;
	TCHAR wname[32];
	HWND hwnd;

	strcpy(wname, YAC_WND_CLASS);

	wcex.cbSize = sizeof(WNDCLASSEX);

	wcex.style			= 0;
	wcex.lpfnWndProc	= (WNDPROC)MainWndProc;
	wcex.cbClsExtra		= 0;
	wcex.cbWndExtra		= 0;
	wcex.hInstance		= yi->hinst;
	wcex.hIcon			= NULL;
	wcex.hCursor		= NULL,
	wcex.hbrBackground	= NULL;
	wcex.lpszMenuName	= NULL;
	wcex.lpszClassName	= wname;
	wcex.hIconSm		= NULL;

	RegisterClassEx(&wcex);

	// Create the window
	hwnd = CreateWindow(wname, "", 0, 0, 0, 0, 0, GetDesktopWindow(),
		NULL, yi->hinst, 0);

	if (hwnd)
	{
		#pragma warning(disable: 4244)
		SetWindowLongPtr(hwnd, GWLP_USERDATA, (LONG_PTR)yi);
		#pragma warning(default: 4244)
	}

	return hwnd;
}


LRESULT CALLBACK MainWndProc(HWND hwnd, UINT msg, WPARAM wparam, LPARAM lparam)
{
	YACINFO *yi = NULL;

	GetYACInfoStruct(&yi);

	if (!yi)
		return DefWindowProc(hwnd, msg, wparam, lparam);

	switch(msg)
	{
	case WM_CREATE:
        yi->uTaskbarReset = RegisterWindowMessage(TEXT("TaskbarCreated"));
        break;

	case WM_TIMER:
		StopNotifyingUser(yi);
		KillTimer(yi->hwnd, TIMER_NOTIFYWND);		
		break;

	case WM_COPYDATA:
		HandleCopyData(hwnd, wparam, lparam, yi);
		return TRUE;

	case YAC_MSG_NOTIFY:
		NotifyClients(yi);
		break;

	case YAC_MSG_NOTIFYUSER:
		if (NotifyUser((TCHAR *)wparam, yi))
		{
			KillTimer(yi->hwnd, TIMER_NOTIFYWND);
			SetTimer(yi->hwnd, TIMER_NOTIFYWND, yi->nDisplayTime, NULL);
		}
		break;

	case YAC_MSG_TRAY:
		switch (lparam)
		{
		case WM_RBUTTONUP:
		case WM_LBUTTONUP:
			ShowContextMenu(hwnd, yi);
			return 0;
		}
		return 0;

	case YAC_MSG_QUIT:
	case WM_DESTROY:
		PostQuitMessage(1);
		break;

	case WM_COMMAND:
		switch(LOWORD(wparam))
		{
		case IDM_ABOUT:
			yi->fAboutDlgShowing = TRUE;
			DialogBox(yi->hinst, MAKEINTRESOURCE(IDD_ABOUT),
				yi->hwnd, AboutDialogProc);
			break;

		case IDM_LISTENERS:
			yi->fListenersDlgShowing = TRUE;
			DialogBox(yi->hinst, MAKEINTRESOURCE(IDD_LISTENERS),
				yi->hwnd, ListenersDialogProc);
			break;

		case IDM_SUBSTITUTIONS:
			yi->fSubsDlgShowing = TRUE;
			DialogBox(yi->hinst, MAKEINTRESOURCE(IDD_SUBSTITUTIONS),
				yi->hwnd, SubsDialogProc);
			break;

		case IDM_FORMATS:
			yi->fFormatsDlgShowing = TRUE;
			DialogBox(yi->hinst, MAKEINTRESOURCE(IDD_FORMATS),
				yi->hwnd, FormatsDialogProc);
			break;

		case IDM_SENDTEXT:
			yi->fSendTextDlgShowing = TRUE;
			DialogBox(yi->hinst, MAKEINTRESOURCE(IDD_SENDTEXT),
				yi->hwnd, SendTextDialogProc);
			break;

		case IDM_DEVICES:
			yi->fStartupSelectDevice = !yi->fStartupSelectDevice;

			if (yi->fStartupSelectDevice)
				SaveInfo("StartupSelectDevice", "1", HKEY_LOCAL_MACHINE);
			else
				SaveInfo("StartupSelectDevice", "0", HKEY_LOCAL_MACHINE);
			break;

		case IDM_OPENLOG:
			ShellExecute(hwnd, "open", yi->szLogFile, NULL, NULL, SW_SHOWNORMAL);
			break;

		case IDM_USEBALLOONS:
			if (yi->uOS >= OS_WINME)
				yi->fUseBalloons = !yi->fUseBalloons;
	
			if (yi->fUseBalloons)
				SaveInfo("UseBalloons", "1", HKEY_LOCAL_MACHINE);
			else
				SaveInfo("UseBalloons", "0", HKEY_LOCAL_MACHINE);
			break;

		case IDM_TRANSLUCENT:
			if (yi->uOS >= OS_WIN2000)
				yi->fTranslucentWindows = !yi->fTranslucentWindows;

			if (yi->fTranslucentWindows)
				SaveInfo("TranslucentWindows", "1", HKEY_LOCAL_MACHINE);
			else
				SaveInfo("TranslucentWindows", "0", HKEY_LOCAL_MACHINE);
			break;

		case IDM_PLAYSOUNDS:
			yi->fPlaySounds = !yi->fPlaySounds;

			if (yi->fPlaySounds)
				SaveInfo("PlaySounds", "1", HKEY_LOCAL_MACHINE);
			else
				SaveInfo("PlaySounds", "0", HKEY_LOCAL_MACHINE);
			break;

		case IDM_TEXTTOLOG:
			yi->fWriteTextToLog = !yi->fWriteTextToLog;
			
			if (yi->fWriteTextToLog)
				SaveInfo("WriteTextToLogFile", "1", HKEY_LOCAL_MACHINE);
			else
				SaveInfo("WriteTextToLogFile", "0", HKEY_LOCAL_MACHINE);
			break;

		case IDM_WRITELOG:
			yi->fWriteLogFile = !yi->fWriteLogFile;

			if (yi->fWriteLogFile)
				SaveInfo("WriteLogFile", "1", HKEY_LOCAL_MACHINE);
			else
				SaveInfo("WriteLogFile", "0", HKEY_LOCAL_MACHINE);
			break;

		case IDM_SHAREMODEM:
			if (ShowShareModemWarning(yi))
			{
				yi->fShareModem = !yi->fShareModem;

				if (yi->fShareModem)
					SaveInfo("ShareModem", "1", HKEY_LOCAL_MACHINE);
				else
					SaveInfo("ShareModem", "0", HKEY_LOCAL_MACHINE);
			}
			break;
			
		case IDM_EXIT:
			if (VerifyExit(yi))
				DestroyWindow(hwnd);
			break;

#ifdef _DEBUG
		case IDM_DEBUG_TESTINCOMING:
			strcpy(yi->szName, TEST_NAME);
			strcpy(yi->szNumber, TEST_NUMBER);
			NotifyClients(yi);
			break;

		case IDM_DEBUG_TESTNETWORK:
			strcpy(yi->szName, TEST_NAME);
			strcpy(yi->szNumber, TEST_NUMBER);
			FormatNameAndNumber(yi->szName, yi->szNumber, yi);
			EncodeCallerIDInfo(yi->szString, yi->szName, yi->szNumber);

			SendMessage(yi->hwnd, YAC_MSG_NOTIFYUSER, (WPARAM)yi->szString, 0);
			yi->szName[0] = '\0';
			yi->szNumber[0] = '\0';
			yi->szString[0] = '\0';
			break;
#endif
		}
		break;

	default:
		if (msg == yi->uTaskbarReset)
			InitSystemTray(yi);
	}

	return DefWindowProc(hwnd, msg, wparam, lparam);
}


BOOL VerifyExit(YACINFO *yi)
{
	TCHAR szText[256];

	LoadString(yi->hinst, IDS_CONFIRMEXIT, szText, 256);
	return (IDYES == MessageBox(yi->hwnd, szText, yi->szAppName,
		MB_ICONQUESTION | MB_YESNO | MB_DEFBUTTON1));
}


void HandleCopyData(HWND hwnd, WPARAM wparam, LPARAM lparam, YACINFO *yi)
{
	TCHAR szString[MSG_LIMIT_CHARS + 1];
	COPYDATASTRUCT *cds = NULL;

	if (yi->nMode == MODE_LISTENER)
		return;

	ZeroMemory(szString, sizeof(szString));

	cds = (COPYDATASTRUCT*)lparam;
	if (cds)
	{
		strncpy(szString, (TCHAR *)cds->lpData, cds->cbData);
		szString[cds->cbData] = '\0';
	}

	if (szString[0] != '\0')
	{
		strncpy(yi->szString, szString, MSG_LIMIT_CHARS);
		yi->szString[MSG_LIMIT_CHARS] = '\0';
		SendNotificationString(yi);
	}
}


BOOL ShowShareModemWarning(YACINFO *yi)
{
	TCHAR szText[256];

	if (yi->fShareModem)
	{
		LoadString(yi->hinst, IDS_RESTART, szText, 256);
		MessageBox(yi->hwnd, szText, yi->szAppName, MB_ICONINFORMATION | MB_OK);

		return TRUE;
	}
	else
	{
		LoadString(yi->hinst, IDS_SHAREWARNING, szText, 256);
		if (IDYES == 
			MessageBox(yi->hwnd, szText, yi->szAppName, MB_ICONSTOP | MB_YESNO | MB_DEFBUTTON2))
		{
			LoadString(yi->hinst, IDS_RESTART, szText, 256);
			MessageBox(yi->hwnd, szText, yi->szAppName, MB_ICONINFORMATION | MB_OK);
			return TRUE;
		}
		else
			return FALSE;
	}
}


void InitYACInfoStruct(YACINFO *yi)
{
	// Everything is initialized to 0 unless explicitly defined here
	ZeroMemory(yi, sizeof(yi));
}


BOOL GetYACInfoStruct(YACINFO **yi)
{
	if (g_yi)
		*yi = g_yi;

	return TRUE;
}


void DoFirstRun(YACINFO *yi)
{
	TCHAR szFormat[64];

	// Put in our default formats, primarily for upgraders
	GetPrivateProfileString(INI_FORMATS, "7", NONESTRING, szFormat, 64, yi->szIniFile);

	if (!strcmp(NONESTRING, szFormat))
		WritePrivateProfileString(INI_FORMATS, "7", DEFAULT_7_FORMAT, yi->szIniFile);

	GetPrivateProfileString(INI_FORMATS, "10", NONESTRING, szFormat, 64, yi->szIniFile);

	if (!strcmp(NONESTRING, szFormat))
		WritePrivateProfileString(INI_FORMATS, "10", DEFAULT_10_FORMAT, yi->szIniFile);
}